'''
Python Cego Driver enables python programs to access cego databases
'''
import cegodb
from ._cegodb import (
    DataError,
    DatabaseError,
    Error,
    IntegrityError,
    InterfaceError,
    InternalError,
    NotSupportedError,
    OperationalError,
    PoolError,
    ProgrammingError,
    Warning,
)

from cegodb.connectionpool import *   # noqa: F401,F403
from cegodb.cursor import Cursor
from cegodb.release_info import __version__ as __version__
from cegodb.release_info import __version_info__ as __version_info__
from cegodb.release_info import __author__ as __author__
from cegodb.connection import Connection

_POOLS = _CONNECTION_POOLS = {}

__all__ = ["DataError", "DatabaseError", "Error", "IntegrityError",
           "InterfaceError", "InternalError", "NotSupportedError",
           "OperationalError", "PoolError", "ProgrammingError",
           "Warning", "Connection", "__version__", "__version_info__",
           "__author__", "Cursor" ]


def connect(*args, connectionclass=cegodb.connection.Connection, **kwargs):
    """
    Creates a Cego Connection object.

    By default the standard connectionclass cegodb.connections.Connection
    will be created.

    Connection parameters are provided as a set of keyword arguments:
        - host:
            The host name or IP address of the database server.
        - user, username:
            The username used to authenticate with the database server
        - password, passwd:
            The password of the given user
        - database, db:
            database (schema) name to use when connecting with the database
            server
        - port:
            port number of the database server. If not specified the default
            value of 3306 will be used.
        - protocol:
            clienter server protocol, depending on the server side protocol setting
            one of : xml, serial, fastserial
        - autocommit=False:
            Specifies the autocommit settings.
            True will enable autocommit, False will disable it (default).

    """

    # print("Python __init__ called\n");
            
    if kwargs:
        if "pool_name" in kwargs:
            if not kwargs["pool_name"] in cego._CONNECTION_POOLS:
                pool = cego.ConnectionPool(**kwargs)
            else:
                pool = cego._CONNECTION_POOLS[kwargs["pool_name"]]
            c = pool.get_connection()
            return c

    connection = connectionclass(*args, **kwargs)
    if not isinstance(connection, cegodb.connection.Connection):
        raise cegodb.ProgrammingError("%s is not an instance of "
                                       "cegodb.Connection" % connection)
    return connection
