#
# Copyright (C) 2024 by Björn Lemke

# This code is derived from MariaDB/Connector written by Georg Richter

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.

# You should have received a copy of the GNU Library General Public
# License along with this library; if not see <http://www.gnu.org/licenses>
# or write to the Free Software Foundation, Inc.,
# 51 Franklin St., Fifth Floor, Boston, MA 02110, USA
#

import cegodb
import socket
import cegodb.cursor

from packaging import version

class Connection(cegodb._cegodb.connection):
    """
    Python Cego Driver Connection Object

    Handles the connection to a Cego database server.
    It encapsulates a database session.

    Connections are created using the method cegodb.connect()
    """

    def _check_closed(self):
        if self._closed:
            raise cegodb.ProgrammingError("Invalid connection or "
                                           "not connected")

    def __init__(self, *args, **kwargs):
        """
        Establishes a connection to a database server and returns a connection
        object.
        """



        # parameters for connection pool
        # not very nice to setup these parameters here
        
        self._used = 0
        self._Connection__pool = None
        self.__last_used = 0

        autocommit = kwargs.pop("autocommit", False)
        
        super().__init__(*args, **kwargs)
        
        self.autocommit = autocommit

    def cursor(self, cursorclass=cegodb.cursor.Cursor, **kwargs):
        """
        Returns a new cursor object for the current connection.

        If no cursorclass was specified, a cursor with default cegodb.Cursor
        class will be created.

        """

        self._check_closed()
        cursor = cursorclass(self, **kwargs)
        if not isinstance(cursor, cegodb._cegodb.cursor):
            raise cegodb.ProgrammingError("%s is not an instance of "
                                           "cegodb.cursor" % cursor)
        return cursor
    
    def close(self):
        self._check_closed()
        if self._Connection__pool:
            self._Connection__pool._close_connection(self)
        else:
            super().close()
            
    def __enter__(self):
        self._check_closed()
        "Returns a copy of the connection."
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._check_closed()
        "Closes connection."
        
        self.close()
        
    def begin(self):
        """
        Start a new transaction which can be committed by .commit() method,
        or cancelled by .rollback() method.
        """
        self._check_closed()
        self._execute_command("start transaction;")
        
    def commit(self):
        """
        Commit any pending transaction to the database.
        """

        self._check_closed()
        self._execute_command("commit;")

    def rollback(self):
        """
        Causes the database to roll back to the start of any pending
        transaction
        """

        self._check_closed()
        self._execute_command("rollback;")

    @property
    def database(self):
        self._check_closed()
        return self._cegodb_get_info("dbname")

    @property
    def database_version(self):
        self._check_closed()
        return self._cegodb_get_info("version")

    
    @database.setter
    def database(self, tableSet):
        """Set default database."""
        self._check_closed()
        self._execute_command("set tableset %s;" % str(tableSet))
        
    @property
    def user(self):
        """
        Returns the user name for the current connection or empty
        string if it can't be determined, e.g. when using socket
        authentication.
        """
        self._check_closed()
        return self._cegodb_get_info("user")

    @property
    def tableset(self):
        """
        Returns the user name for the current connection or empty
        string if it can't be determined, e.g. when using socket
        authentication.
        """
        self._check_closed()
        return self._cegodb_get_info("tableset")

    @property
    def server_port(self):
        """
        Database server TCP/IP port. This value will be 0 in case of a unix
        socket connection.
        """

        self._check_closed()
        return self._cegodb_get_info("port")


    @property
    def server_name(self):
        """Name or IP address of database server."""

        self._check_closed()
        return self._cegodb_get_info("host")


    @property
    def autocommit(self):
        """
        Toggles autocommit mode on or off for the current database connection.

        Autocommit mode only affects operations on transactional table types.
        Be aware that rollback() will not work, if autocommit mode was switched
        on.

        By default autocommit mode is set to False."
        """

        self._check_closed()
        return False
        # bool(self.server_status & STATUS.AUTOCOMMIT)

    @autocommit.setter
    def autocommit(self, mode):
        self._check_closed()
        if bool(mode) == self.autocommit:
            return

        if mode:
            self._execute_command("set autocommit on;")
        else:
            self._execute_command("set autocommit off;")

    @property
    def open(self):
        """
        Returns true if the connection is alive.

        A ping command will be send to the server for this purpose,
        which means this function might fail if there are still
        non processed pending result sets.
        """

        self._check_closed()
        try:
            self.ping()
        except cegodb.Error:
            return False
        return True

    @property
    def thread_id(self):
        """
        Alias for connection_id
        """

        self._check_closed()
        return self.connection_id
