#
# Copyright (C) 2024 by Björn Lemke

# This code is derived from MariaDB/Connector written by Georg Richter

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.

# You should have received a copy of the GNU Library General Public
# License along with this library; if not see <http://www.gnu.org/licenses>
# or write to the Free Software Foundation, Inc.,
# 51 Franklin St., Fifth Floor, Boston, MA 02110, USA
#

import cegodb
import datetime
from numbers import Number
from typing import Sequence
import collections.abc

ROWS_EOF = -1

class Cursor(cegodb._cegodb.cursor):
    """
    Cego Connector Cursor Object
    """

    def check_closed(self):
        if self.closed:
            self._connection._check_closed()
            raise cegodb.ProgrammingError("Cursor is closed")

    def __init__(self, connection, **kwargs):
        """
        initialization
        """
        self._connection = connection
        self._rowcount = 0
        self._data = None

        if not connection:
            raise cegodb.ProgrammingError("Invalid or no connection provided")


        # call initialization of main class
        super().__init__(connection, **kwargs)


    def callproc(self, sp: str, data: Sequence = ()):
        """
        Executes a stored procedure sp. The data sequence must contain an
        entry for each parameter the procedure expects.

        Input/Output or Output parameters have to be retrieved by .fetch
        methods, the .sp_outparams attribute indicates if the result set
        contains output parameters.

        Arguments:
            - sp: Name of stored procedure.
            - data: Optional sequence containing data for placeholder
                    substitution.
        """

        self.check_closed()

        """
        We have to check, data parameter is of sequence type
        For procedures with one parameter, data is given with parameter type        
        """
        
        if isinstance(data, collections.abc.Sequence) :
            params = ""
            if data and len(data):
                params = ("?," * len(data))[:-1]
        else:
            params ="?"

        
        # create statement
        statement = "call %s(%s);" % (sp, params)
        self._spname = sp
        self._statement = statement
        self._data = data
        self._rowcount = 0

        self._executeproc()

        
    def execute(self, statement: str, data: Sequence = (), buffered=None):
        
        self.check_closed()
        
        #
        # for data parameter, we need a case handling
        # in case of empty parameter list, or parameter list > 1
        # a sequence is given ( object type = tuple )
        # If a single parameter is given, the seems to be converted to a single object value
        # Strange ...
        # Anyway, a further case handling then is done CegoCursor_bindQuery routine
        #
        
        if isinstance(data, collections.abc.Sequence) :
            # print("Sequence Data")
            # for i in range(0, len(data)):
            #    print("XXX Data %d = %s" % ( i, data[i] ))
            self._numparam = len(data)
        else:
            # print("Single Data %s" %  data )
            self._numparam = 1
            
        self._data=data
            
        # elif data is None:
        #    print("Data is none")
        # normData.append(data)

        # print("Statement is  %s" % statement )
        
        self._statement = statement;
        self._preparequery()
        self._bindquery()
        self._executequery()
        self._initresult()                    

    def executemany(self, statement, parameters):
        """
        Prepare a database operation (INSERT,UPDATE,REPLACE or DELETE
        statement) and execute it against all parameter found in sequence.

        Exactly behaves like .execute() but accepts a list of tuples, where
        each tuple represents data of a row within a table.
        .executemany() only supports DML (insert, update, delete) statements.

        If the SQL statement contains a RETURNING clause, executemany()
        returns a result set containing the values for columns listed in the
        RETURNING clause.
        """
        self.check_closed()

        if not parameters or not len(parameters):
            raise cegodb.ProgrammingError("No data provided")

        self._statement = statement;

        self._preparequery()

        for data in parameters:

            if isinstance(data, collections.abc.Sequence) :
                self._numparam = len(data)
            else:
                self._numparam = 1

            self._data = data
            self._bindquery()

            self._executequery()

    def close(self):
        """
        Closes the cursor.

        If the cursor has pending or unread results, .close() will cancel them
        so that further operations using the same connection can be executed.

        The cursor will be unusable from this point forward; an Error
        (or subclass) exception will be raised if any operation is attempted
        with the cursor."
        """

        # CONPY-231: fix memory leak
        if self._data:
            del self._data

        # if not self.connection._closed:
        super().close()

    def fetchone(self):
        """
        Fetch the next row of a query result set, returning a single sequence,
        or None if no more data is available.

        An exception will be raised if the previous call to execute() didn't
        produce a result set or execute() wasn't called before.
        """

        if not self.field_count:
            raise cegodb.ProgrammingError("Cursor doesn't have a result set")
        row = super().fetchone()

        return row

    def fetchmany(self, size: int = 0):
        """
        Fetch the next set of rows of a query result, returning a sequence
        of sequences (e.g. a list of tuples). An empty sequence is returned
        when no more rows are available.

        The number of rows to fetch per call is specified by the parameter.
        If it is not given, the cursor's arraysize determines the number
        of rows to be fetched. The method should try to fetch as many rows
        as indicated by the size parameter.
        If this is not possible due to the specified number of rows not being
        available, fewer rows may be returned.

        An exception will be raised if the previous call to execute() didn't
        produce a result set or execute() wasn't called before.
        """
        if not self.buffered:
            self.check_closed()

        if size == 0:
            size = self.arraysize

        return super().fetchrows(size)

    def fetchall(self):
        """
        Fetch all remaining rows of a query result, returning them as a
        sequence of sequences (e.g. a list of tuples).

        An exception will be raised if the previous call to execute() didn't
        produce a result set or execute() wasn't called before.
        """
            
        self.check_closed()
        return super().fetchrows(ROWS_EOF)

    def __iter__(self):
        return iter(self.fetchone, None)

    def setinputsizes(self, size: int):
        """
        Required by PEP-249. Does nothing in Cego driver
        """
        return

    def setoutputsize(self, size: int):
        """
        Required by PEP-249. Does nothing in Cego driver
        """
        return

    def __enter__(self):
        """ 
        Returns a copy of the cursor
        """
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """
        Closes cursor
        """
        self.close()

    @property
    def rowcount(self):
        """
        This read-only attribute specifies the number of rows that the last\
        execute*() produced (for DQL statements like SELECT) or affected
        (for DML statements like UPDATE or INSERT).
        The return value is -1 in case no .execute*() has been performed
        on the cursor or the rowcount of the last operation  cannot be
        determined by the interface.
        """

        # Even if PEP-249 permits operations on a closed cursor, we don't
        # raise an exception if the cursor or the underlying connection
        # was closed (See CONPY-269), instead we will return -1

        if self._rowcount > 0:
            return self._rowcount
        return super().rowcount

    @property
    def sp_outparams(self):
        """
        Returns procedure out parameters
        """
        self.check_closed()
        return self._outparam

    @property
    def connection(self):
        """
        Read-Only attribute which returns the reference to the connection
        object on which the cursor was created.
        """
        return self._connection
